@@ -22,6 +22,7 @@
 #include <QTemporaryDir>
 #include <QTemporaryFile>
 
+#include <kwidgetsaddons_version.h>
 #include <KPluginMetaData>
 #include <KActionCollection>
 #include <KJobWidgets>
@@ -560,41 +561,14 @@ void KomparePart::saveDiff()
         saveProperties(config.data());
         config->sync();
 
-        while (1)
-        {
-            QUrl url = QFileDialog::getSaveFileUrl(widget(), i18nc("@title:window", "Save .diff"),
-                                                   m_info.destination,
-                                                   i18n("Patch Files (*.diff *.dif *.patch)"));
-            if (url.isLocalFile() && QFile::exists(url.toLocalFile())) {
-                int result = KMessageBox::warningYesNoCancel(widget(), i18n("The file exists or is write-protected; do you want to overwrite it?"), i18nc("@window:title", "File Exists"), KStandardGuiItem::overwrite(), KGuiItem(i18nc("@action:button", "Do Not Overwrite")));
-                if (result == KMessageBox::Cancel)
-                {
-                    break;
-                }
-                else if (result == KMessageBox::No)
-                {
-                    continue;
-                }
-                else
-                {
-                    qCDebug(KOMPAREPART) << "URL = " << url.toDisplayString();
-                    qCDebug(KOMPAREPART) << "Directory = " << w->directory();
-                    qCDebug(KOMPAREPART) << "DiffSettings = " << m_diffSettings;
+        QUrl url = QFileDialog::getSaveFileUrl(widget(), i18nc("@title:window", "Save .diff"),
+                                               m_info.destination,
+                                               i18n("Patch Files (*.diff *.dif *.patch)"));
+        qCDebug(KOMPAREPART) << "URL = " << url.toDisplayString();
+        qCDebug(KOMPAREPART) << "Directory = " << w->directory();
+        qCDebug(KOMPAREPART) << "DiffSettings = " << m_diffSettings;
 
-                    m_modelList->saveDiff(url.url(), w->directory(), m_diffSettings);
-                    break;
-                }
-            }
-            else
-            {
-                qCDebug(KOMPAREPART) << "URL = " << url.toDisplayString();
-                qCDebug(KOMPAREPART) << "Directory = " << w->directory();
-                qCDebug(KOMPAREPART) << "DiffSettings = " << m_diffSettings;
-
-                m_modelList->saveDiff(url.url(), w->directory(), m_diffSettings);
-                break;
-            }
-        }
+        m_modelList->saveDiff(url.url(), w->directory(), m_diffSettings);
     }
 }
 
@@ -676,8 +650,8 @@ void KomparePart::onContextMenuRequested(const QPoint& pos)
 
 void KomparePart::updateCaption()
 {
-    QString source = m_info.source.toDisplayString();
-    QString destination = m_info.destination.toDisplayString();
+    QString source = m_info.source.toDisplayString(QUrl::PreferLocalFile);
+    QString destination = m_info.destination.toDisplayString(QUrl::PreferLocalFile);
 
     QString text;
 
@@ -701,8 +675,8 @@ void KomparePart::updateCaption()
 
 void KomparePart::updateStatus()
 {
-    QString source = m_info.source.toDisplayString();
-    QString destination = m_info.destination.toDisplayString();
+    QString source = m_info.source.toDisplayString(QUrl::PreferLocalFile);
+    QString destination = m_info.destination.toDisplayString(QUrl::PreferLocalFile);
 
     QString text;
 
@@ -775,7 +749,11 @@ void KomparePart::slotSwap()
 {
     if (m_modelList->hasUnsavedChanges())
     {
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        int query = KMessageBox::warningTwoActionsCancel
+#else
         int query = KMessageBox::warningYesNoCancel
+#endif
                     (
                         widget(),
                         i18n("You have made changes to the destination file(s).\n"
@@ -785,7 +763,11 @@ void KomparePart::slotSwap()
                         KStandardGuiItem::discard()
                     );
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        if (query == KMessageBox::PrimaryAction)
+#else
         if (query == KMessageBox::Yes)
+#endif
             m_modelList->saveAll();
 
         if (query == KMessageBox::Cancel)
@@ -806,20 +788,29 @@ void KomparePart::slotRefreshDiff()
 {
     if (m_modelList->hasUnsavedChanges())
     {
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        int query = KMessageBox::warningTwoActionsCancel
+#else
         int query = KMessageBox::warningYesNoCancel
+#endif
                     (
                         widget(),
                         i18n("You have made changes to the destination file(s).\n"
                              "Would you like to save them?"),
                         i18nc("@title:window", "Save Changes?"),
                         KStandardGuiItem::save(),
-                        KStandardGuiItem::discard()
+                        KStandardGuiItem::discard(),
+                        KStandardGuiItem::cancel()
                     );
 
         if (query == KMessageBox::Cancel)
             return; // Abort prematurely so no refreshing
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+        if (query == KMessageBox::PrimaryAction)
+#else
         if (query == KMessageBox::Yes)
+#endif
             m_modelList->saveAll();
     }
 
@@ -921,20 +912,29 @@ bool KomparePart::queryClose()
 {
     if (!m_modelList->hasUnsavedChanges()) return true;
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    int query = KMessageBox::warningTwoActionsCancel
+#else
     int query = KMessageBox::warningYesNoCancel
+#endif
                 (
                     widget(),
                     i18n("You have made changes to the destination file(s).\n"
                          "Would you like to save them?"),
                     i18nc("@title:window", "Save Changes?"),
                     KStandardGuiItem::save(),
-                    KStandardGuiItem::discard()
+                    KStandardGuiItem::discard(),
+                    KStandardGuiItem::cancel()
                 );
 
     if (query == KMessageBox::Cancel)
         return false;
 
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    if (query == KMessageBox::PrimaryAction)
+#else
     if (query == KMessageBox::Yes)
+#endif
         return m_modelList->saveAll();
 
     return true;
